package com.persagy.dmp.middleware.controller;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.FloorDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.service.IFloorService;
import com.persagy.dmp.middleware.service.ISpaceService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import com.persagy.dmp.middleware.utils.SortUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.management.ObjectInstance;
import javax.ws.rs.QueryParam;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName FloorController
 * @Description: 楼层
 * @Author linhuili
 * @Date 2021/9/9 10:24
 * @Version V1.0
 **/
@RestController
@RequestMapping("floor")
public class FloorController {

    @Autowired
    private IFloorService floorService;

    @Autowired
    private IBaseService baseService;

    @Autowired
    private ISpaceService spaceService;

    /**
     * 查询建筑下的楼层列表
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/queryList")
    public CommonResult queryList(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getBuildId())){
            throw new Exception("建筑ID不能为空");
        }
        // 条件转换
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria,RelationTypeEnum.ARCHSUBSET_BD2FL.getGraphCode(),
                RelationTypeEnum.ARCHSUBSET_BD2FL.getRelCode(), request.getBuildId(), 0);
        // 查询建筑下的楼层列表
        CommonResult response = baseService.doQuery(context,queryCriteria ,null);
        List<JSONObject> content = (List<JSONObject>) response.getContent();
        //结果集处理
        handleResultList(content);
        response.setContent(content);
        return  response;
    }

    /**
     * 编辑楼层信息
     * @param projectId
     * @param list
     * @return
     * @throws Exception
     */
    @PostMapping("/save")
    public CommonResult update(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                                  @RequestBody List<FloorDTO> list) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        CommonResult<Object> response = new CommonResult<>();
        //保存楼层信息
        baseService.doUpdate(context,null,list);
        response.setContent(list);
        return  response;
    }


    /**
     * 结果集处理
     * @param content
     * @return
     */
    private void handleResultList(List<JSONObject> content){
        if(CollUtil.isEmpty(content)){
           return ;
        }
        for (JSONObject node : content) {
            Double area = node.getDouble("area");
            if(area == null){
                node.put("area",0);
            }
            Double height = node.getDouble("height");
            if(height == null){
                node.put("height",0);
            }
        }
    }

    /**
     * 统计楼层下各信息
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/count")
    public CommonResult count(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getFloorId())){
            throw new Exception("楼层ID不能为空");
        }
        // 统计楼层下各信息
        CommonResult response = floorService.count(context,request);
        return  response;
    }

    /**
     *  查询/统计楼层下的设备，空间，阀门
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public CommonResult query(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getBuildId())){
            throw new Exception("建筑ID不能为空");
        }
        //查询/统计楼层下的设备空间
        List<JSONObject> content = floorService.query(context,request);

        //类型名称相同的合并处理
        content = EntityTransferUtil.combineData(content);
        //按首字母，对象数量为0放后的规则进行排序
        content = SortUtil.sortByCharacter(content, CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(content);
        response.setCount(content == null ? 0 : Long.valueOf(content.size()));
        return  response;
    }



    /**
     * 查询楼层下的空间列表
     * @param projectId
     * @param list
     * @return
     * @throws Exception
     */
    @PostMapping("/queryFloorSpaceList")
    public CommonResult queryFloorUnSpaceList(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                                            @RequestBody List<CommonRequestDTO> list) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(CollUtil.isEmpty(list)){
            return new CommonResult();
        }
        List<JSONObject> result = new ArrayList<>();
        for (CommonRequestDTO commonRequestDTO : list) {
            if(StringUtils.isEmpty(commonRequestDTO.getFloorId())){
                throw new Exception("楼层ID不能为空");
            }
            List<JSONObject> content = floorService.queryFloorUnSpaceList(context, commonRequestDTO);
            JSONObject floorSpace = new JSONObject();
            floorSpace.put("floorId",commonRequestDTO.getFloorId());
            if(CollUtil.isNotEmpty(content)){
                //类型名称相同的合并处理
                content = EntityTransferUtil.combineData(content);
                //按首字母，对象数量为0放后的规则进行排序
                content = SortUtil.sortByCharacter(content, CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
            }
            floorSpace.put("spaceInfos",content);
            result.add(floorSpace);
        }
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(result);
        response.setCount(result == null ? 0 : Long.valueOf(result.size()));
        return  response;
    }
}
